# -*- coding: utf-8 -*-
from django.db import models
from django.contrib.auth.models import User

# Create your models here.
# added and modified fields are not defined for Language (really doesn't matter), Tags (overkill), EntityType, WordGrammarType, ConceptDescription.
class Language(models.Model):
	localized_name = models.CharField(max_length=200)
	english_name = models.CharField(max_length=200)
	iso_code = models.CharField(max_length=7)
	def __unicode__(self):
		return self.localized_name

class Tag(models.Model):
	name = models.CharField(max_length=200)
	upstream = models.ManyToManyField('self')
	upstream.symmetric = False
	upstream.blank = True
	upstream.null = True
	equal = models.ManyToManyField('self')
	equal.symmetric = True
	equal.blank = True
	equal.null = True
	added = models.DateTimeField()
	added.auto_now_add = True
	modified = models.DateTimeField()
	modified.auto_now = True
	def __unicode__(self):
		return self.name

class EntityType(models.Model):
	name = models.CharField(max_length=20)
	description = models.CharField(max_length=100)
	description.blank = True
	description.null = True
	# We will have built-in list of entity types, because it would be
	# overkill for now to have them editable and translatable
	def __unicode__(self):
		return self.name

class WordGrammarType(models.Model):
	name = models.CharField(max_length=20)
	description = models.CharField(max_length=100)
	description.blank = True
	description.null = True
	abbr = models.CharField(max_length=20)
	sequence = models.IntegerField()
	def __unicode__(self):
		return self.name

class Word(models.Model):
	word = models.CharField(max_length=200)
	language = models.ForeignKey(Language)
	phonetic_transcription = models.CharField(max_length=200)
	phonetic_transcription.blank = True
	phonetic_transcription.null = True
	#grammar_type = models.ForeignKey(WordGrammarType)
	#grammar_type.blank = True
	#grammar_type.null = True
	no_back_translation = models.ManyToManyField("self")
	no_back_translation.symmetrical = False
	no_back_translation.blank = True
	no_back_translation.null = True
	#comments = models.ManyToManyField(Comment)
	#comments.blank = True
	#comments.null = True
	#comments - see Comment object, related_name
	added = models.DateTimeField()
	added.auto_now_add = True
	modified = models.DateTimeField()
	modified.auto_now = True
	def __unicode__(self):
		return self.word

class Entity(models.Model):
	name = models.CharField(max_length=500)
	short_name = models.CharField(max_length=100)
	type = models.ForeignKey(EntityType)
	added = models.DateTimeField()
	added.auto_now_add = True
	modified = models.DateTimeField()
	modified.auto_now = True
	def __unicode__(self):
		return self.name

class Concept(models.Model):
	comment = models.CharField(max_length=2000)
	comment.blank = True
	comment.null = True
	tags = models.ManyToManyField(Tag)
	tags.blank = True
	tags.null = True
	#comments = models.ManyToManyField(Comment)
	#comments.blank = True
	#comments.null = True
	#bindings = models.ManyToManyField(Binding)
	#bindings.blank = True
	#bindings.null = True
	grammar_type = models.ForeignKey(WordGrammarType)
	grammar_type.blank = True
	grammar_type.null = True
	added = models.DateTimeField()
	added.auto_now_add = True
	modified = models.DateTimeField()
	modified.auto_now = True
	def __unicode__(self):
		return self.comment

class Binding(models.Model):
	word = models.ForeignKey(Word)
	concept = models.ForeignKey(Concept, related_name = "bindings")
	tags = models.ManyToManyField(Tag)
	tags.blank = True
	tags.null = True
	entities = models.ManyToManyField(Entity)
	entities.blank = True
	entities.null = True
	added = models.DateTimeField()
	added.auto_now_add = True
	modified = models.DateTimeField()
	modified.auto_now = True
	def __unicode__(self):
		return self.word

class ConceptDescription(models.Model):
	concept = models.ForeignKey(Concept, related_name = "descriptions")
	title = models.CharField(max_length=200)
	title.blank = True
	title.null = True
	description = models.CharField(max_length=2000)
	language = models.ForeignKey(Language)
	def __unicode__(self):
		return self.title

class UserData(models.Model):
	user = models.ForeignKey(User, unique = True)
	language = models.ForeignKey(Language)
	view = models.CharField(max_length=20)
	view.blank = True
	view.null = True
	subscribed_concepts = models.ManyToManyField(Concept, related_name="subscribed_users")
	def __unicode__(self):
		return self.user.username

class Comment(models.Model):
	comment_subject = models.CharField(max_length=100)
	comment_subject.blank = True
	comment_subject.null = True
	comment = models.CharField(max_length=10000)
	user = models.ForeignKey(User)
	concept = models.ForeignKey(Concept, related_name = 'comments')
	concept.blank = True
	concept.null = True
	word = models.ForeignKey(Word, related_name = 'comments')
	word.blank = True
	word.null = True
	added = models.DateTimeField()
	added.auto_now_add = True
	modified = models.DateTimeField()
	modified.auto_now = True
	def __unicode__(self):
		return self.comment
