# -*- coding: utf-8 -*-
import django.forms as forms
from django.core.exceptions import *
from wordy.database.models import *
from sets import Set
import re

def findUILanguage():
   try:
       return Language.objects.get(iso_code__iexact = 'LV')
   except Language.DoesNotExist:
       return None

def getOrCreateTag(tagName):
    #finds exsting tag or creates new one

   try:
       return Tag.objects.get(name = tagName)
   except Tag.DoesNotExist:
        tag = Tag()
        tag.name = tagName
        tag.save()
        return tag

DEFAULT_UI_LANGUAGE = findUILanguage()
PAGE_SIZE = 100
GOOD_SEARCH_RESULTS = 50
NEEDSREVIEW_TAG = 'needsreview'

class ConceptData:
    def __init__(self):
        self.isFuzzy = False
        self.concept = Concept()
        self.description = None
        self.fullBindings = []
        self.bindings = []
        self.sourceBindings = []
        self.comments = []
        self.commentCount = 0
        self.tags = []
        self.isSubscribed = False

    def copy(self):
        result = ConceptData()
        result.isFuzzy = self.isFuzzy
        result.concept = self.concept
        result.description = self.description
        result.fullBindings = self.fullBindings
        result.bindings = []
        result.sourceBindings = []
        result.comments = self.comments
        result.commentCount = self.commentCount
        result.tags = self.tags
        result.isSubscribed = self.isSubscribed
        return result

class WordData:
    def __init__(self, word):
        self.concepts = []
        self.word = word

class UserForm(forms.ModelForm):

    class Meta:
        model = User
        fields = ('comment','grammar_type')

class DescriptionForm(forms.ModelForm):
    description = forms.CharField(widget = forms.Textarea )

    class Meta:
        model = ConceptDescription
        fields = ('title','description', 'language')

class TagForm(forms.ModelForm):
    class Meta:
        model = Tag
        fields = ('name')

class UserForm(forms.ModelForm):
    class Meta:
        model = User

class UserDataForm(forms.ModelForm):
    class Meta:
        model = UserData


class ListPage:
    batchSize = PAGE_SIZE

    def __init__(self, objList, offset):
        self.offset = int(offset)
        self.nextOffset = self.offset + PAGE_SIZE
        self.prevOffset = self.offset - PAGE_SIZE

        tempList = objList[self.offset: self.nextOffset + 1]
        self.hasPrev = self.prevOffset >= 0
        self.hasNext = len(list(tempList)) == PAGE_SIZE + 1

        if (self.hasNext):
            tempList = tempList[0: PAGE_SIZE]
        self.data = tempList


def getUserLanguage(user):
    #TODO:
    return DEFAULT_UI_LANGUAGE
    if user.is_authenticated() :
        return user.get_profile().language
    else :
        return DEFAULT_UI_LANGUAGE



def getSearchResultsType(request):
    if request.user.is_authenticated() :
        if len(request.user.get_profile().view) > 0:
            return request.user.get_profile().view

    if "searchResultsType" in request.session :
        return request.session["searchResultsType"]
    else :
        return "terms"

def setSearchResultsType(request, newType):
    if request.user.is_authenticated() :
        profile = request.user.get_profile()
        profile.view  = newType
        profile.save()
    else :
        request.session["searchResultsType"] = newType


def getUserForeignLanguage(user):
    return Language.objects.filter(iso_code__iexact = 'en')[0:1].get()



def searchExistingTags(searchQuery, minLen):
    result = []
    words = searchQuery.split(',');

    for word in words :
        word = word.strip()
        if len(word) >= minLen :
            tagList = Tag.objects.filter(name__startswith = word)
            result.extend(tagList)
        try:
            tag = Tag.objects.get(name = word)
            if not result.contains(tag):
                result.append(tag)
        except:
            pass

    return result



#this method relies on that binding list has selected related concept, word and word__language
def makeBindingListTranslations(bindingList, user, getComments):
    conceptList = []
    
    for binding in bindingList:
        conceptList.append(binding.concept)

    conceptResults = getConceptListDataBasic(conceptList, user, getComments)

    conceptIdToData={}
    for conceptResult in conceptResults:
        conceptIdToData[conceptResult.concept.id] = conceptResult

    #glue concepts back to their original bindings
    results = []
    takenConcepts = {}
    for binding in bindingList:
        conceptResult = conceptIdToData[binding.concept_id]

        if conceptResult in takenConcepts:
            conceptResult = conceptResult.copy()
        else:
            takenConcepts[conceptResult] = True
        
        conceptResult.sourceBindings.append(binding)

        for targetBinding in conceptResult.fullBindings:
            if targetBinding.word.language != binding.word.language :
                conceptResult.bindings.append(targetBinding)

        results.append(conceptResult)

    return results


def makeConceptListTranslations(conceptList, user, getComments):
    sourceLang = getUserLanguage(user)
    results = getConceptListDataBasic(conceptList, user, getComments)
    
    for conceptResult in results:
        # collect bindings
        for binding in conceptResult.fullBindings:
            if (binding.word.language == sourceLang):
                conceptResult.sourceBindings.append(binding)
            else:
                conceptResult.bindings.append(binding)
    
    return results


def makeConceptListDetails(conceptList, user):
    results = getConceptListDataBasic(conceptList, user, True)

    for conceptResult in results:
        conceptResult.bindings = conceptResult.fullBindings
        
    return results



def getConceptListDataBasic(conceptList, user, getComments):
    userLanguage = getUserLanguage(user)

    conceptIds = []
    for concept in conceptList: conceptIds.append(concept.pk)
    
    concepts = Concept.objects.filter(pk__in=conceptIds).select_related("grammar_type")

    conceptMap = {}
    results = []
    for concept in concepts:
    
        conceptResult = ConceptData()
        conceptResult.concept = concept
        conceptResult.isSubscribed = False
        conceptResult.isFuzzy = False
        #django has no outer join, so prepare tags in simple way
        conceptResult.tags = Tag.objects.filter(concept = concept).order_by("name")

        conceptMap[concept.id] = conceptResult
        results.append(conceptResult)

    #prepare subscribed_concepts
    if user.is_authenticated():
        subscribedConcepts = Concept.objects.filter(pk__in=conceptIds).filter(subscribed_users = user.get_profile())
        for concept in subscribedConcepts:
            conceptResult = conceptMap[concept.id]
            conceptResult.isSubscribed = True;

    #prepare fuzzy concepts
    fuzzyConcepts = Concept.objects.filter(pk__in=conceptIds).filter(tags = getOrCreateTag(NEEDSREVIEW_TAG))
    for concept in fuzzyConcepts:
        conceptResult = conceptMap[concept.id]
        conceptResult.isFuzzy = True;

    # prepare descriptions
    descriptions = ConceptDescription.objects.filter(concept__id__in=conceptIds).select_related("language", "concept")
    for description in descriptions:
        conceptResult = conceptMap[description.concept.id]
        if description.language == userLanguage:
            conceptResult.description = description
        elif conceptResult.description == None:
            conceptResult.description = description
    #fill concepts with no descriptions
    for conceptResult in results:
        if conceptResult.description == None:
           conceptResult.description = ConceptDescription()
           conceptResult.description.title = conceptResult.concept.comment

    # fill concepts with bindings
    bindings = Binding.objects.filter(concept__id__in = conceptIds).select_related("word", "word__language").order_by('word__language__iso_code','word__word')
    prepareBindingUsages(bindings)

    for binding in bindings:
        conceptResult = conceptMap[binding.concept_id]
        conceptResult.fullBindings.append(binding)

    if (getComments):
        comments = Comment.objects.filter(concept__id__in=conceptIds).select_related("user").order_by('added')
        for comment in comments:
            conceptResult = conceptMap[comment.concept_id]
            conceptResult.comments.append(comment)
            conceptResult.commentCount = conceptResult.commentCount + 1

    return results


#builds a usage list for all bindings with a small number of queries
def prepareBindingUsages(bindingList):

    #get unique bindings
    bindingMap = {}
    keyList = []
    for binding in bindingList:
        if not binding.id in bindingMap:
            binding.usageList = []
            bindingMap[binding.id] = binding
            keyList.append(binding.id)

    #get all entities involved
    entities = Entity.objects.filter(binding__in = bindingList).order_by("name").distinct()

    # get bindings for each entity
    for entity in entities:
        bindings = Binding.objects.filter(pk__in = keyList).filter(entities = entity)
        for binding in bindings:
            bindingMap[binding.id].usageList.append(entity)
