# -*- coding: utf-8 -*-
import django.forms as forms
from django.core.mail import send_mail
from django.core.exceptions import *
from wordy.database.models import *
from sets import Set
from prepareData import *
from django.template import loader, Context
from django.conf import settings
import re


class ConceptForm(forms.ModelForm):
    class Meta:
        model = Concept
        fields = ('comment','grammar_type')

def getOrNoneLanguage(langId) :
    try:
        return Language.objects.get(pk = langId)
    except ObjectDoesNotExist:
        return None

def getOrNoneGrammarType(grammarTypeId):
    try:
        return WordGrammarType.objects.get(pk = grammarTypeId)
    except ObjectDoesNotExist:
        return None

def getOrNoneProject(projectId):
    try:
        return Entity.objects.get(pk = projectId)
    except ObjectDoesNotExist:
        return None


def getOrCreateWord(lang, name):
    #finds exsting word or creates new one
    words = Word.objects.filter(word = name)
    if lang :
        words = words.filter(language = lang)

    if words.count() > 0 :
        return words[0]
    elif lang :
        word = Word()
        word.word = name
        word.language = lang
        word.save()
        return word
    else:
        return None

def getOrNoneTag(tagName):
    #tests if there exists tag with given name, returns None if not
    tags = Tag.objects.filter(name = tagName)
    if tags.count() > 0 :
        return tags[0]
    else :
        return None

def setSubscribedConcept(user, concept, isSubscribed):
    if isSubscribed and user.get_profile().subscribed_concepts.filter(pk=concept.pk).count() == 0:
        user.get_profile().subscribed_concepts.add(concept)
    elif not isSubscribed and user.get_profile().subscribed_concepts.filter(pk=concept.pk).count() > 0:
        user.get_profile().subscribed_concepts.remove(concept)


def addNewComment(user, concept, caption, text):
    #adds new comment and sends out mail messages
    comment = Comment()
    comment.comment_subject = caption
    comment.comment = text
    comment.concept = concept
    comment.user = user
    comment.save()

    template = loader.get_template('emailNotification.txt')

    for mailToUser in concept.subscribed_users.all():
        description = getConceptDescription(concept, mailToUser)
        email = mailToUser.user.email
        name = mailToUser.user.first_name
        if len(email) > 0:
            subject = u"[Vārdenes terminu apspriešana] komentārs pie " + description.title
            message = template.render(Context({'author':user, 'reciever': mailToUser.user, 'comment':comment, 'concept': concept, 'description':description, 'urlBase':settings.BASE_URL }))
            send_mail(
                subject,
                message,
                'noreply@vardene.lv',
                [email],
                fail_silently=False)


def joinConcepts(conceptList):
    #joins all concepts into one
    result = conceptList[0]

    boundWords = []
    for binding in result.bindings.all():
        boundWords.append(binding.word)

    for pos in range(1, len(conceptList)):
        concept = conceptList[pos]

        #copy bindings
        for binding in concept.bindings.all():
            if not (binding.word in boundWords):
                binding.concept = result
                boundWords.append(binding.word)
                binding.save()
        #copy comments
        for comment in concept.comments.all():
            comment.concept = result
            if comment.comment_subject != None :
                comment.comment_subject = "(Apvienots no cita termina) " + comment.comment_subject
            else:
                comment.comment_subject = "(Apvienots no cita termina)"
            comment.save()

        #copy descriptions
        for description in concept.descriptions.all():
            description.concept = result
            description.save()

        concept.delete()

    result.tags.add(getOrCreateTag(NEEDSREVIEW_TAG))
    result.save()
    return result


#retuns the new concept if one was created
def conceptReviewAction(concept, request):

    action = request.POST['action']

    if action == "delete" :
        concept.delete()
    elif (action=='save'):
        form = ConceptForm(request.POST, instance=concept)
        form.save()

        if ("reviewed" in request.POST):
            concept.tags.remove(getOrCreateTag(NEEDSREVIEW_TAG))
        else:
            concept.tags.add(getOrCreateTag(NEEDSREVIEW_TAG))

        #descriptions
        #exisiting
        for description in concept.descriptions.all() :
            descId = description.id

            if not ('description' + str(descId) + '_title' in request.POST):
                description.delete()
            else:
                descLan = int(request.POST['description' + str(descId) + '_language' ])
                descTitle = request.POST['description' + str(descId) + '_title' ]
                descDesc = request.POST['description' + str(descId) + '_description' ]
                description.language = getOrNoneLanguage(descLan)
                description.title = descTitle
                description.description = descDesc
                description.save()
        #new
        for attr in request.POST.keys() :
            match = re.match(r'^newdescription(?P<desc_id>\d+)_title$', attr)
            if match == None:
                continue
            descId = int(match.group('desc_id'))

            descLan = int(request.POST['newdescription' + str(descId) + '_language' ])
            descTitle = request.POST['newdescription' + str(descId) + '_title' ]
            descDesc = request.POST['newdescription' + str(descId) + '_description' ]
            description = ConceptDescription()
            description.language = getOrNoneLanguage(descLan)
            description.title = descTitle
            description.description = descDesc
            description.concept = concept
            description.save()

        # translations
        #exsisting
        for binding in concept.bindings.all() :
            bindingId = binding.id
            if not ('binding' + str(bindingId) + '_word' in request.POST):
                binding.delete()
            else:
                bindingLanId = int(request.POST['binding' + str(bindingId) + '_language' ])
                bindingWord = request.POST['binding' + str(bindingId) + '_word' ]
                bindingLan = getOrNoneLanguage(bindingLanId)

                if len(bindingWord) == 0 or bindingLan == None :
                    binding.delete()
                else:
                    binding.word = getOrCreateWord(bindingLan, bindingWord)
                    binding.save()

        # new
        for attr in request.POST.keys() :
            match = re.match(r'^newbinding(?P<word_id>\d+)_word$', attr)
            if match == None:
                continue

            wordId = int(match.group('word_id'))
            wordText = request.POST['newbinding' + str(wordId) + '_word' ]
            wordLanId = int(request.POST['newbinding' + str(wordId) + '_language' ])
            wordLan = getOrNoneLanguage(wordLanId)

            word = getOrCreateWord(wordLan, wordText)
            if word:
                binding = Binding()
                binding.word = word
                binding.concept = concept
                binding.save()


    elif action == 'addDesc' :
        newDesc = ConceptDescription()
        newDesc.concept = concept
        newDesc.language = getUserLanguage(request.user)
        newDesc.save()
    elif (action=='deleteSel'):
        #descriptions
        for description in concept.descriptions.all() :
            descId = description.id
            selected = ('description' + str(descId) + '_selected') in request.POST
            if selected :
                description.delete()

        #delete bindings
        for binding in concept.bindings.all() :
            bindingId = binding.id
            selected = ('binding' + str(bindingId) + '_selected') in request.POST
            if selected :
                binding.delete()
    elif (action=='splitSel'):

        newConcept = Concept()
        newConcept.comment = ' '
        newConcept.grammar_type = concept.grammar_type
        newConcept.save()

        for description in concept.descriptions.all():
            newDesc = ConceptDescription()
            newDesc.language = description.language
            newDesc.title = description.title
            newDesc.description = description.description
            newDesc.concept = newConcept
            newDesc.save()

        for binding in concept.bindings.all():
            bindingId = binding.id
            copy = ('binding' + str(bindingId) + '_copy') in request.POST
            delete = ('binding' + str(bindingId) + '_selected') in request.POST

            if copy and delete :
                binding.concept = newConcept
                binding.save()
            elif copy :
                newBinding = Binding()
                newBinding.concept = newConcept
                newBinding.word = binding.word
                newBinding.save()
                for tag in binding.tags.all():
                    newBinding.tags.add(tag)
                for ent in binding.entities.all():
                    newBinding.entities.add(ent)
                newBinding.save()
        return newConcept


def joinTags(tagList, newTagName):
    # collect concepts that are tagged with one of the tags
    concepts = Set()
    for tag in tagList :
        for concept in tag.concept_set.all():
            concepts.add(concept)

    # create the new tag
    newTag = Tag()
    newTag.name = newTagName
    newTag.save()

    # assign all the concepts
    for concept in concepts:
        newTag.concept_set.add(concept)
    newTag.save()

    # delete the old tags
    for tag in tagList:
        tag.delete()
       
    return newTag


            