# -*- coding: utf-8 -*-
import django.forms as forms
from django.core.exceptions import *
from wordy.database.models import *
from sets import Set
from prepareData import *
import re



def searchConcepts(wordText, tagFilter, user):
    #returns list of WordData
    foundWords = searchWordsInt(wordText, tagFilter)
    if foundWords.count() > 200:
        foundWords = foundWords[0:200]
    result = prepareSearchResults(foundWords, user)
    return result

class ResultsByLang:
    def __init__(self, lang):
        self.lang = lang
        self.results = []

def searchConceptsByLang(wordText, tagFilter, user):
    #returns list of ResultsByLang, containing ConceptData objects
    foundWords = searchWordsInt(wordText, tagFilter).select_related("language")
    if foundWords.count() > 200:
        foundWords = foundWords[0:200]

    userLang = getUserLanguage(user)

    #sort by language
    wordsByLang = {}
    langs = []
    for word in foundWords:
        language = word.language
        if not language in wordsByLang :
            wordsByLang[language] = []
            langs.append(language)
        wordsByLang[language].append(word)

    # sort so that native language comes last
    langs.sort(lambda x,y: languageOrderSorter(x,y, userLang))

    result = []
    for lang in langs :
        pack = ResultsByLang(lang)
        pack.results = prepareConceptSearchResults(wordsByLang[lang], user)
        result.append(pack)

    return result


def searchWordsByLang(wordText, tagFilter, user):
    #returns list of ResultsByLang, containing WordData objects
    foundWords = searchWordsInt(wordText, tagFilter).select_related("language")
    if foundWords.count() > 200:
        foundWords = foundWords[0:200]

    userLang = getUserLanguage(user)

    #sort by language
    wordsByLang = {}
    langs = []
    for word in foundWords:
        language = word.language
        if not language in wordsByLang :
            wordsByLang[language] = []
            langs.append(language)
        wordsByLang[language].append(word)

    # sort so that native language comes last
    langs.sort(lambda x,y: languageOrderSorter(x,y, userLang))

    result = []
    for lang in langs :
        pack = ResultsByLang(lang)
        pack.results = prepareWordSearchResults(wordsByLang[lang], user)
        result.append(pack)

    return result


def languageOrderSorter(lang1, lang2, userLang):
    if (lang1 == lang2):
        return 0
    elif (lang1 == userLang):
        return 1
    elif (lang2 == userLang):
        return -1
    else:
        return cmp(lang1.iso_code, lang2.iso_code)


def prepareConceptSearchResults(wordList, user):
    result = getWordListData(wordList, user, False)
    result.sort(searchResultWordSorter)

    #merge results with the same concept
    conceptMap = {}
    result1 = []
    for data in result:
        if (data.concept in conceptMap):
            data1 = conceptMap[data.concept]
            data1.sourceBindings.extend(data.sourceBindings)
        else:
            conceptMap[data.concept] = data
            result1.append(data)

    return result1

def prepareWordSearchResults(wordList, user):
    result = getWordListData(wordList, user, False)
    result.sort(searchResultWordSorter)

    #merge results with the same word
    wordMap = {}
    result1 = []
    for data in result:
        word = data.sourceBindings[0].word
        if (word in wordMap):
            wordObj = wordMap[word]
        else:
            wordObj = WordData(word)
            wordMap[word] = wordObj
            result1.append(wordObj)
        wordObj.concepts.append(data)

    for wordObj in result1 :
        wordObj.concepts.sort(conceptDataPartOfSpeechSorter)

    return result1


def conceptDataPartOfSpeechSorter(data1, data2):
    pos1 = data1.concept.grammar_type
    pos2 = data2.concept.grammar_type

    if pos1 == None:
        return 1
    elif pos2 == None:
        return -1
    else :
        return cmp(pos1.name, pos2.name)

    return 0

def getWordListData(wordList, user, getComments):
    bindings = Binding.objects.filter(word__in=wordList).select_related("concept", "word", "word__language")    
    return makeBindingListTranslations(bindings, user, getComments)

def searchResultWordSorter(w1,w2):
    if w1.isFuzzy and not w2.isFuzzy:
        return 1
    elif not w1.isFuzzy and w2.isFuzzy:
        return -1
    else :
        return len(w1.sourceBindings[0].word.word) - len(w2.sourceBindings[0].word.word)


#returns a list of words
def searchWordsInt(wordText, tagFilter):
    baseWords = Word.objects.all()
    # this is clever - we try to minmize search queries

    words = baseWords.filter(word__istartswith = wordText)
    if (tagFilter):
        words = tagFilter.filterWordList(words)
    words = filterOrphanedWords(words)

    if len(words) <= GOOD_SEARCH_RESULTS:
        #startswith was too little extend with words that contain words that start with and return
        wordsPP = baseWords.filter(word__icontains = " " + wordText)
        if (tagFilter):
            wordsPP = tagFilter.filterWordList(wordsPP)
        wordsPP = filterOrphanedWords(wordsPP)

        #concatentate the results
        return words | wordsPP
    else:
        #startswith was too big, try something preciser
        prevWords = words
        
    words = baseWords.filter(word__iexact = wordText)
    if (tagFilter):
        words = tagFilter.filterWordList(words)
    words = filterOrphanedWords(words)

    if len(words) <= GOOD_SEARCH_RESULTS:
        return prevWords
    prevWords = words


    words = baseWords.filter(word = wordText)
    if (tagFilter):
       exactWords = tagFilter.filterWordList(words)
    words = filterOrphanedWords(words)

    if len(words) <= GOOD_SEARCH_RESULTS:
        return prevWords
    prevWords = words

    return prevWords


def filterOrphanedWords(wordSet):
    return wordSet.filter(binding__id__isnull = False)
    #result = []
    #for word in wordSet:
    #    if word.binding_set.count() > 0:
    #        result.append(word)
    #return result
