# -*- coding: utf-8 -*-
from django.http import HttpResponse, HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response as render_to_response_django

from django.template import RequestContext

import django.contrib.auth as auth
from django.contrib.auth.forms import *
from django.contrib.auth.decorators import login_required
import django.forms
from django.conf import settings


from tagfilter import *
from wordy.database.models import *
from prepareData import *
from exportTBX import *
from processing import *
from search import *


import re

def render_response(request, template, variables = {}):
    #add additional template variables
    if "message" in request.session :
        variables["message"] = request.session["message"]
        del request.session["message"]
    if TagFilter.TAGFILTER_VAR_INC in request.session :
        variables[TagFilter.TAGFILTER_VAR_INC] = request.session[TagFilter.TAGFILTER_VAR_INC]
    if TagFilter.TAGFILTER_VAR_EX in request.session :
        variables[TagFilter.TAGFILTER_VAR_EX] = request.session[TagFilter.TAGFILTER_VAR_EX]

    return render_to_response_django(template, variables, context_instance=RequestContext(request))

def redirect_to_next(request):
    if "next" in request.REQUEST :
        return HttpResponseRedirect(request.REQUEST["next"])
    elif "next" in request.session :
        next = request.session["next"]
        del request.session["next"]
        return HttpResponseRedirect(next)
    else :
        return HttpResponse("empty")

def redirect_to_next_or_other(request, other):
    if "next" in request.REQUEST :
        return HttpResponseRedirect(request.REQUEST["next"])
    elif "next" in request.session :
        next = request.session["next"]
        del request.session["next"]
        return HttpResponseRedirect(next)
    else :
        return HttpResponseRedirect(other)

# Create your views here.

def main(request):
    return render_response(request, 'main.html')

def about(request):
    return render_response(request, 'about.html')

def systeminfo(request):
    nrTag = getOrCreateTag(NEEDSREVIEW_TAG)

    userCount = User.objects.all().count()
    conceptCount = Concept.objects.all().count()
    needsReviewCount = Concept.objects.filter(tags = nrTag).count()

    wordCount = Word.objects.all().count()

    return render_response(request, 'systeminfo.html', {'userCount': userCount, 'conceptCount':conceptCount, 'needsReviewCount':needsReviewCount, 'wordCount':wordCount })

def tbxExport(request):
    exportData = prepareDataForExport()

    return render_response(request, 'tbxexport.xml', {'concepts': exportData})

def login(request):
    authForm = AuthenticationForm(request.GET, request.POST)
    if 'message' in request.session:
        del request.session["message"]

    if authForm.is_valid() :
        data = authForm.clean()
        user = authForm.get_user()
        auth.login(request, user)
        return redirect_to_next(request)
    else :
        request.session["message"] = "Neizdevās pieteikties"
        return redirect_to_next(request)

def logout(request):
     auth.logout(request)
     return redirect_to_next_or_other(request, reverse('main', args=()))

def register(request):
    if (request.method == "GET"):
        languages = Language.objects.all()
        return render_response(request, 'user_register.html',  {'languages': languages, 'defaultLanguage':findUILanguage() })
    else :
        loginName = request.POST["username"]
        passw = request.POST["password"]
        passw2 = request.POST["password2"]
        lang = int(request.POST["language"])
        name = request.POST['name']
        email = request.POST['email']
        error = None
        message = None

        if len(loginName) == 0 :
            error = "Ievadiet lietotājvārdu"
        elif User.objects.filter(username = loginName).count() > 0 :
            error = "Šāds lietotājs jau eksistē, izvēlieties citu"
        elif len(passw) < 5:
            error = "Parolei vajag vismaz 5 simbolus"
        elif passw != passw2:
            error = "Paroles nesakrīt"
        else:
            user = User.objects.create_user(loginName, email, passw)
            user.first_name = name
            user.save()
            profile = UserData()
            profile.language = Language.objects.get(pk = lang)
            profile.view=""
            profile.user = user
            profile.save()
            message = "Veiksmīgi reģistrēts"

        languages = Language.objects.all()
        return render_response(request, 'user_register.html',  {'errorMessage':error, 'message':message, 'languages': languages, 'defaultLanguage':findUILanguage() })

        languages = Language.objects.all()
        return render_response(request, 'user_register.html',  {'errorMessage':error, 'languages': languages, 'defaultLanguage':findUILanguage() })

@login_required
def user(request):

    if (request.method == "GET"):
        userF = UserForm(instance = request.user)
        userDataF = UserDataForm(instance = request.user.get_profile())
        return render_response(request, 'userPage.html', {'form':userF, 'dataForm':userDataF})
    else:
        user = request.user
        action = request.REQUEST['action']

        if action == 'password':
            oldPass = request.REQUEST['passwordOld']
            newPass = request.REQUEST['password1']
            checkPass = request.REQUEST['password2']

            if not user.check_password(oldPass):
                message = "Nepareiza vecā parole"
            elif (newPass != checkPass):
                message = "Jaunās paroles nesakrīt"
            else:
                message = "Parole nomainīta"
                user.set_password(newPass)
                user.save()

            request.session['message'] = message
        elif action == 'save':
            userData = instance = request.user.get_profile()
            userF = UserForm(request.POST)
            userDataF = UserDataForm(request.POST)
            user.first_name = userF.data['first_name']
            user.email = userF.data['email']
            userData.language = Language.objects.get(pk = int(userDataF.data['language']))

            user.save();
            userData.save();
        elif action == 'delete':
            if 'confirmed' in request.POST:
                auth.logout(request)
                user.delete()
                message = "Lietotājs izdzēsts, lai jums laba diena"
                request.session['message'] = message
                return HttpResponseRedirect(reverse('main', args=()))
            else:
                message = "Dzēšana netika apstiprināta"

            request.session['message'] = message
        else:
            request.session['message'] = "No action specified"

        return HttpResponseRedirect(reverse('userPage', args=()))

def search(request, offset):
    query = request.REQUEST['q'].replace("+"," ")
    message = ''

    if (query == ""):
        return HttpResponseRedirect(reverse('main', args=()))

    resultsType = getSearchResultsType(request)

    if resultsType == "dict":
        results = searchWordsByLang(query, TagFilter(request), request.user)
    else:
        results = searchConceptsByLang(query, TagFilter(request), request.user)

    if (len(results) == 0):
        message = 'Nekas netika atrasts'

    tags = Tag.objects.all()
    nativeLang = getUserLanguage(request.user)
    foreignLang = getUserForeignLanguage(request.user)
    languages = Language.objects.all()
    grammarTypes = WordGrammarType.objects.order_by('sequence')
    projects = Entity.objects.all()

    total = 0
    for langRes in results: total += len(langRes.results)

    if resultsType == "terms" :
        return render_response(request, 'searchResults_terms.html', {
            'resultsType' : resultsType,
            'resultsByLang': results,
            'query':query,
            'message': message,
            'allTags':tags,
            'count': total,
            'grammarTypes': grammarTypes,
            'languages':languages,
            'targetLanguage':foreignLang,
            'projectList': projects})
    else :
        return render_response(request, 'searchResults_dict.html', {
            'resultsType' : resultsType,
            'resultsByLang': results,
            'query':query,
            'message': message,
            'allTags':tags,
            'count': total,
            'grammarTypes': grammarTypes,
            'languages':languages,
            'targetLanguage':foreignLang,
            'projectList': projects})

def switchSearchResultsType(request):
    newType = request.REQUEST["type"]
    setSearchResultsType(request, newType)
    return redirect_to_next(request)

def opensearchdescription(request):
    response = HttpResponse(mimetype='application/opensearchdescription+xml')
    t = loader.get_template('opensearch.xml')
    c = Context({'urlBase':settings.BASE_URL})
    response.write(t.render(c))
    return response

def opensearch(request):
    query = request.REQUEST['q'].replace("+"," ")
    words = searchWords(query, None)
    words = sortWords(words)

    response = HttpResponse(mimetype='application/x-suggestions+json')
    t = loader.get_template('opensearch_suggestions.xml')
    c = Context({'query':query, 'words':words, 'urlBase':settings.BASE_URL})
    response.write(t.render(c))
    return response


def wordList0(request):
    return wordList(request, 0)

def wordList(request, offset):
    words = Word.objects
    words = TagFilter(request).filterWordList(words)

    page = ListPage(words, offset)
    page.data = getWordListData(page.data, True)
    tags = Tag.objects.all()
    return render_response(request, 'word_list.html', {'list': page, 'allTags': tags})

@login_required
def wordListAction(request):

    #collect words
    wordList = []
    for attr in request.POST.keys() :
        match = re.match(r'^word(?P<word_id>\d+)_selected$', attr)
        if match:
            wordId = int(match.group('word_id'))
            word = Word.objects.get(id = wordId)
            wordList.append(word)

    action = request.POST['action']

    # get tag if needed
    if action == 'removeTag':
        tagName = request.POST['tag_name']
        tag = getOrNoneTag(tagName)
    elif action == 'addTag' :
        tagName = request.POST['tag_name'];
        tag = getOrCreateTag(tagName)


    if action == 'join' :
        conceptList = Concept.objects.filter(bindings__word__in = wordList).all()
        concept = joinConcepts(conceptList)
        return HttpResponseRedirect(reverse('conceptDetails', args=[concept.id]))
    else:
        for word in wordList:
            if action == 'delete' :
                #deletes also all concepts
                for concept in Concept.objects.filter(bindings__word = word).all():
                    concept.delete()
                word.delete()
            elif action == 'removeTag' and tag != None :
                for binding in word.binding_set.all() :
                    concept = binding.concept
                    concept.tags.remove(tag)
                    concept.save()
            elif action == 'addTag' :
                for binding in word.binding_set.all() :
                    concept = binding.concept
                    concept.tags.add(tag)
                    concept.save()

    return redirect_to_next(request)


#show all translations, except from word base language
def wordDetails(request, word_id):
    word = get_object_or_404(Word, pk=word_id)
    preparedConcepts = makeBindingListTranslations(word.binding_set.all(), request.user, True)

    return showWordDetails(request, word, preparedConcepts)

#show only translations to particular language
def wordTranslations(request, word_id, language_id):
    #TODO: this does not work as needed, remove or fix
    return None

def showWordDetails(request, word, concepts):
    nativeLang = getUserLanguage(request.user)
    foreignLang = getUserForeignLanguage(request.user)
    languages = Language.objects.all()
    grammarTypes = WordGrammarType.objects.order_by('sequence')
    projects = Entity.objects.all()

    if word.language == nativeLang:
        targetLang = foreignLang
    else:
        targetLang = nativeLang

    return render_response(request, 'word_translations.html', {
        'word': word,
        'grammarTypes': grammarTypes,
        'languages':languages,
        'targetLanguage':targetLang ,
        'conceptList': concepts,
        'projectList': projects,
        'multipleConcepts': len(concepts) > 1})



@login_required
def wordAdd(request):
    wordText = request.REQUEST["word"]
    language_id = int(request.REQUEST["lang"])
    lang = Language.objects.get(pk = language_id)
    word = getOrCreateWord(lang, wordText)
    return HttpResponse(str(word.pk))

@login_required
def wordDelete(request, word_id):
    word = get_object_or_404(Word, pk=word_id)
    word.delete();
    return redirect_to_next_or_other(request, reverse('main', args=()))

@login_required
def wordReview(request, word_id):
    word = get_object_or_404(Word, pk=word_id)
    #TODO:

    return redirect_to_next_or_other(request, reverse('main', args=()))


def conceptDetails(request, concept_id):
    concept = get_object_or_404(Concept, pk=concept_id)
    conceptData = makeConceptListDetails([concept], request.user)[0]
    grammarTypes = WordGrammarType.objects.order_by('sequence')

    languages = Language.objects.all()
    projects = Entity.objects.all()
    return render_response(request, 'concept_detail.html', {
        'conceptData': conceptData,
        'languages':languages,
        'projectList': projects,
        'grammarTypes': grammarTypes})

@login_required
def conceptTagAdd(request):
    concept = get_object_or_404(Concept, pk=int(request.REQUEST["conceptId"]))

    if len(request.REQUEST["tagId"]) > 0 and int(request.REQUEST["tagId"])!= -1 :
        tag_id = int(request.REQUEST["tagId"])
        tag = get_object_or_404(Tag, pk=tag_id)
    else:
        tagName =request.REQUEST["tagName"]
        tag = getOrCreateTag(tagName)

    concept.tags.add(tag);
    concept.save();

    conceptData = makeConceptListDetails([concept], request.user)[0]
    if "ajax" in request.REQUEST:
        return render_response(request, 'concept_tagBar_inc.html', {'ajax':True, 'concept':concept, 'conceptData':conceptData})
    else:
        return redirect_to_next(request)

@login_required
def conceptTagRemove(request):
    concept = get_object_or_404(Concept, pk=int(request.REQUEST["conceptId"]))
    tag = get_object_or_404(Tag, pk=int(request.REQUEST["tagId"]))
    concept.tags.remove(tag);
    concept.save();

    conceptData = makeConceptListDetails([concept], request.user)[0]
    if "ajax" in request.REQUEST:
        return render_response(request, 'concept_tagBar_inc.html', {'ajax':True, 'concept':concept, 'conceptData':conceptData})
    else:
        return redirect_to_next(request)

@login_required
def conceptDelete(request, concept_id):
    concept = get_object_or_404(Concept, pk=concept_id)
    concept.delete();
    return HttpResponseRedirect(reverse('main', args=()))

@login_required
def conceptAddComment(request, concept_id):
    concept = get_object_or_404(Concept, pk=concept_id)
    text = request.POST['comment']
    caption = request.POST['caption']
    subscribe = 'subscribe' in request.POST
    setSubscribedConcept(request.user, concept, subscribe )

    addNewComment(request.user, concept, caption, text)
    return redirect_to_next(request)

def conceptSetSubscribtion(request, concept_id):
    concept = get_object_or_404(Concept, pk=concept_id)
    subscribe = 'subscribe' in request.REQUEST
    setSubscribedConcept(request.user, concept, subscribe )
    return redirect_to_next(request)


@login_required
def user_concepts_review(request, offset):
    needsreview = getOrCreateTag(NEEDSREVIEW_TAG)
    usertag = getOrCreateTag('user:' + request.user.username)
    filter = TagFilter()
    filter.addFilter(needsreview)
    filter.addFilter(usertag)
    offset = int(offset)

    if 'next' in request.REQUEST:
        next = request.REQUEST['next']
    else:
        next = reverse("userPage", args=[])

    if request.method == "GET" or request.POST['action'] == "revert":
        #render the inital page
        conceptList = Concept.objects
        conceptList = filter.filterConceptList(conceptList)
        total = conceptList.count()

        if ((offset < 0) or (offset >= total)):
            return redirect_to_next_or_other(request, reverse("userPage", args=[]))
        else:
            concept = conceptList[offset]
            needsReview = getOrCreateTag(NEEDSREVIEW_TAG) in concept.tags.all()
            cform = ConceptForm(instance=concept)
            grammarTypes = WordGrammarType.objects.order_by('sequence')
            languages = Language.objects.all()
            return render_response(request, 'tag_conceptsReview.html', {'next':next, 'offset':offset, 'total':total, 'concept': concept,'form':cform, 'grammarTypes': grammarTypes, 'isReviewed':not needsReview ,'languages': languages})
    else:
        #perform action
        action = request.POST['action']
        conceptId = int(request.POST['concept'])
        concept = get_object_or_404(Concept, pk=conceptId)

        newConcept = conceptReviewAction(concept, request)

        if action=='save' :
            if not 'reviewed' in request.POST :
                offset = offset + 1
        elif action == 'delete' :
            total = total -1
        elif action == 'addDesc' :
            #nothing here
            notAA = 1
        elif action == 'previous' :
            offset = offset - 1
        elif action=='deleteSel' :
            #nothing here
            notAA = 1
        elif action=='splitSel' :
            #nothing here
            notAA = 1

        #render response
        conceptList = Concept.objects
        conceptList = filter.filterConceptList(conceptList)
        total = conceptList.count()

        #redirect to next concept
        if ((offset < 0) or (offset >= total)):
            return redirect_to_next_or_other(request, reverse("userPage", args=[]))
        elif action=='splitSel' :
            # render both current and the split concept
            needsReview = getOrCreateTag(NEEDSREVIEW_TAG) in concept.tags.all()
            cform = ConceptForm(instance=concept)
            grammarTypes = WordGrammarType.objects.order_by('sequence')
            languages = Language.objects.all()
            return render_response(request, 'tag_conceptsReview.html', {'next':next, 'offset':offset, 'total':total, 'concept': concept, 'isReviewed':not needsReview , 'hasNew':True,'newConcept':newConcept ,'form':cform, 'grammarTypes': grammarTypes, 'languages': languages})
        else:
            return HttpResponseRedirect(reverse('user_concepts_review', args=[offset]))

@login_required
def conceptBulkAdd(request):
    '''Page to create new concept and some initial translations.
    '''
    if request.method == "GET" :
        languages = Language.objects.all()
        nativeLang = getUserLanguage(request.user)
        foreignLang = getUserForeignLanguage(request.user)
        grammarTypes = WordGrammarType.objects.order_by('sequence')
        projects = Entity.objects.all()
        return render_response(request, 'concept_bulkAdd.html', {
            'languages':languages,
            'sourceLanguage':foreignLang,
            'targetLanguage':nativeLang,
            'grammarTypes': grammarTypes,
            'projectList': projects})

    action = request.POST['action']

    if action == 'save':
        #save concepts
        #TODO:
        a = 1

    else:
        return HttpResponseRedirect(reverse('main', args=[]))


@login_required
def conceptNew(request):
    '''Page to create new concept and some initial translations.
    '''
    if request.method == "GET" :
        languages = Language.objects.all()
        nativeLang = getUserLanguage(request.user)
        foreignLang = getUserForeignLanguage(request.user)
        grammarTypes = WordGrammarType.objects.order_by('sequence')
        projects = Entity.objects.all()

        return render_response(request, 'concept_new.html', {
            'languages':languages,
            'sourceLanguage':foreignLang,
            'targetLanguage':nativeLang,
            'grammarTypes': grammarTypes,
            'projectList': projects})

    action = request.POST['action']

    if action == 'save':
        #save concept
        concept = Concept()
        concept.comment = request.POST['title']
        grammarTypeId = int(request.POST['grammarType'])
        concept.grammar_type = getOrNoneGrammarType(grammarTypeId)
        concept.save()

        desc = ConceptDescription()
        desc.concept = concept
        desc.title = request.POST['title']
        desc.description = request.POST['description']
        desc.language = getUserLanguage(request.user)
        desc.save()

        projectId = int(request.POST['sourceProject'])
        project = getOrNoneProject(projectId)

        #set needsreview
        if not request.user.is_staff:
            desc.tags.add(getOrCreateTag(NEEDSREVIEW_TAG))

        #Save translations

        for wordAttr in request.POST.keys() :
            match = re.match(r'^word_(?P<word_no>\d+)_word$', wordAttr)
            if match == None :
                continue
            wordNo = int(match.group('word_no'))
            langAttr = 'word_' + str(wordNo) + '_language'

            wordText = request.POST[wordAttr]

            if len(wordText) == 0:
                continue

            wordLang = getOrNoneLanguage(int(request.POST[langAttr]))

            if (len(wordText) > 0) and (wordLang):
                word = getOrCreateWord(wordLang, wordText)
                binding = Binding()
                binding.concept = concept
                binding.word = word
                binding.save()

                if (project != None):
                    binding.entities.add(project)
                    binding.save()

        return HttpResponseRedirect(reverse('conceptDetails', args=[concept.id]))
    else:
        return HttpResponseRedirect(reverse('main', args=[]))

@login_required
def conceptListAction(request):
    action = request.POST['action']

    #collect concepts
    conceptList = []
    for attr in request.POST.keys() :
        match = re.match(r'^concept(?P<concept_id>\d+)_selected$', attr)
        if match == None :
            continue
        conceptId = int(match.group('concept_id'))
        concept = get_object_or_404(Concept, pk = conceptId)
        conceptList.append(concept)

    #get additional objects
    if action == 'removeTag':
        tagName = request.POST['tag_name']
        tag = getOrNoneTag(tagName)
    elif action == 'addTag' :
        tagName = request.POST['tag_name'];
        tag = getOrCreateTag(tagName)

    #perform action
    if action == 'join':
        joinConcepts(conceptList)
    else:
        for concept in conceptList:
            if action  == 'delete' :
                concept.delete();
            elif action == 'removeTag' and tag != None :
                concept.tags.remove(tag)
                concept.save()
            elif action == 'addTag' :
                concept.tags.add(tag)
                concept.save()

    return redirect_to_next(request)


@login_required
def conceptAddBinding(request, concept_id):
    conceptObj = get_object_or_404(Concept, pk=concept_id)

    if "wordId" in request.REQUEST:
        word_id = int(request.REQUEST["wordId"])
        wordObj = get_object_or_404(Word, pk=word_id)
    else:
        wordText = request.REQUEST["word"]
        langId = int(request.REQUEST["language"])
        wordLang = get_object_or_404(Language, pk= langId)
        wordObj = getOrCreateWord(wordLang, wordText)

    existingBinding = Binding.objects.filter(concept = concept_id).filter(word = wordObj)

    if (existingBinding.count() == 0) :
        binding = Binding()
        binding.concept = conceptObj
        binding.word = wordObj
        binding.save()

    if "next" in request.REQUEST:
        return redirect_to_next(request)
    else:
        return HttpResponse("DONE");

@login_required
def bindingSetUses(request):
    binding_id = int(request.REQUEST['objectId'])
    bindingObj = get_object_or_404(Binding, pk=binding_id)

    if request.method== "GET":       #render the usages part to show
        projects = Entity.objects.all()
        prepareBindingUsages([bindingObj])
        return render_response(request, 'binding_usages_inc.html', {'showEdit':True, 'binding':bindingObj, 'projectList':projects})
    elif request.method== "POST":         #save results and show new state
        #collect projects
        projectList = []
        for attr in request.POST.keys() :
            match = re.match(r'^project_(?P<id>\d+)_selected$', attr)
            if match == None :
                continue
            projectId = int(match.group('id'))
            project = get_object_or_404(Entity, pk = projectId)
            projectList.append(project)
        bindingObj.entities = projectList
        bindingObj.save()

        prepareBindingUsages([bindingObj])

        if "ajax" in request.REQUEST:
            return render_response(request, 'binding_usages_inc.html', {'newResult':True, 'binding':bindingObj})
        else:
            return redirect_to_next(request)

def addBindingSearch(request):
    query = request.REQUEST['q']
    words = Word.objects.filter(word__contains=query)

    if "lang" in request.REQUEST :
        language = Language.objects.get(pk = int(request.REQUEST["lang"]))
        words = words.filter(language = language)
    words = words.order_by('language__iso_code','word')[0: 10]
    return render_response(request, 'addBindingSearch.html', {'words': words})

def bindingDelete(request, binding_id):
    binding = get_object_or_404(Binding, pk=binding_id)
    binding.delete()

    return redirect_to_next(request)



@login_required
def descriptionDelete(request, description_id):
    description = get_object_or_404(ConceptDescription, pk = description_id)
    description.delete();
    return redirect_to_next(request)

@login_required
def descriptionEdit(request, description_id):
    descObj = get_object_or_404(ConceptDescription, pk = description_id)

    if request.method == "GET":
        languages = Language.objects.all()
        return render_response(request, 'concept_description_edit_inc.html', {'showEdit':True,'description': descObj, 'languages': languages})
    else :  #save the result
        descObj.title = request.POST["title"]
        descObj.language = get_object_or_404(Language, pk=int(request.POST["language"]))
        descObj.description = request.POST["description"]
        descObj.save();

        if "ajax" in request.REQUEST:
            return render_response(request, 'concept_description_edit_inc.html', {'newResult':True, 'description':descObj})
        else:
            return redirect_to_next(request)

@login_required
def descriptionAdd(request, concept_id):
    concept = get_object_or_404(Concept, pk = concept_id)

    if request.method == "GET":
        languages = Language.objects.all()
        form = DescriptionForm()
        return render_response(request, 'description_add.html', {'concept': concept, 'form':form, 'languages': languages, 'next': nextUrl})
    else :
        action = request.POST["action"]
        if (action == "save") :
            descObj = ConceptDescription()
            descObj.concept = concept
            descObj.title = request.POST["title"]
            descObj.language = get_object_or_404(Language, pk=int(request.POST["language"]))
            descObj.description = request.POST["description"]
            descObj.save();
            return redirect_to_next(request)
        elif (action == "cancel"):
            return redirect_to_next(request)
        else :
            return render_response(request, 'description_add.html', {'concept': concept, 'form':form, 'languages': languages, 'next': nextUrl})

@login_required
def conceptGrammarTypeSet(request, concept_id):
    concept = get_object_or_404(Concept, pk=concept_id)

    if request.method == "GET":
         grammarTypes = WordGrammarType.objects.order_by('sequence')
         return render_response(request, 'concept_grammarTypeBar_inc.html', {'concept': concept, 'showEdit':True, 'grammarTypes': grammarTypes})
    else:
        grammarType = get_object_or_404(WordGrammarType, pk=request.REQUEST['grammarType'])
        concept.grammar_type = grammarType
        concept.save();

        if "ajax" in request.REQUEST:
            return render_response(request, 'concept_grammarTypeBar_inc.html', {'newResult':True, 'concept':concept})
        else:
            return redirect_to_next(request)

@login_required
def conceptReview(request, concept_id):
    concept = get_object_or_404(Concept, pk=concept_id)

    if request.method == "GET" or request.POST['action'] == "revert":
        if 'next' in request.REQUEST:
            request.session['next'] = request.REQUEST['next']
        needsReview = getOrCreateTag(NEEDSREVIEW_TAG) in concept.tags.all()
        cform = ConceptForm(instance=concept)
        grammarTypes = WordGrammarType.objects.order_by('sequence')
        languages = Language.objects.all()
        needsReview = getOrCreateTag(NEEDSREVIEW_TAG) in concept.tags.all()
        return render_response(request, 'concept_review.html', {'concept': concept,'form':cform, 'isReviewed':not needsReview, 'grammarTypes': grammarTypes, 'languages': languages})

    action = request.POST['action']

    newConcept = conceptReviewAction(concept, request)

    if action =='cancel':
        return redirect_to_next_or_other(request, reverse('conceptDetails', args=[concept_id]))
    if action=='save' :
        return redirect_to_next_or_other(request, reverse('conceptDetails', args=[concept_id]))
    elif action == 'delete' :
        return redirect_to_next_or_other(request, reverse('main', args=[]))
    elif action == 'addDesc' :
        return HttpResponseRedirect(reverse('conceptReview', args=[concept_id]))
    elif action == 'addDesc' :
        return HttpResponseRedirect(reverse('conceptReview', args=[concept_id]))
    elif (action=='deleteSel'):
        return HttpResponseRedirect(reverse('conceptReview', args=[concept_id]))
    elif (action=='splitSel'):
        # render both
        needsReview = getOrCreateTag(NEEDSREVIEW_TAG) in concept.tags.all()
        cform = ConceptForm(instance=concept)
        grammarTypes = WordGrammarType.objects.order_by('sequence')
        languages = Language.objects.all()
        return render_response(request, 'concept_review.html', {'concept': concept, 'hasNew':True,'newConcept':newConcept ,'form':cform, 'isReviewed':not needsReview, 'grammarTypes': grammarTypes, 'languages': languages})



def tagList(request):
    list = Tag.objects.all()
    return render_response(request, 'tagListBase.html', {'list': list})

@login_required
def tagListAction(request):
    action = request.POST['action']
    tagList = []

    for attr in request.POST.keys() :
        match = re.match(r'^tag(?P<tag_id>\d+)_selected$', attr)
        if match == None :
            continue

        tagId = int(match.group('tag_id'))
        tag = Tag.objects.get(id = tagId)
        tagList.append(tag)

    if action == 'delete' :
        for tag in tagList :
            tag.delete()
    elif action == 'join':
        newTagName = request.POST['name']
        joinTags(tagList, newTagName)

    return redirect_to_next(request)


@login_required
def tagAdd(request):
    nextUrl = ""
    if ('next' in request.REQUEST) :
        nextUrl = request.REQUEST['next']

    if request.method == "GET":
        form = TagForm()
        return render_response(request, 'tag_add.html', {'form':form, 'next': nextUrl})
    else :
        action = request.POST["action"]
        if (action == "save") :
            form = TagForm(request.POST)

            if (form.is_valid()):
                tag = form.save()
                return HttpResponseRedirect(nextUrl)
            else :
                return render_response(request, 'tag_add.html', {'form':form, 'next': nextUrl})
        elif (action == "cancel"):
            return redirect_to_next(request)
        else :
            return render_response(request, 'tag_add.html', {'form':form, 'next': nextUrl})


@login_required
def tagDelete(request, tag_id):
    tag = get_object_or_404(Tag, pk=tag_id)
    tag.delete();
    return redirect_to_next_or_other(request, reverse('tagList', args=()))

def tagDetails(request, tag_id, offset):
    tag  = get_object_or_404(Tag, pk=tag_id)

    allConcepts = tag.concept_set.all()
    allConcepts = TagFilter(request).filterConceptList(allConcepts.filter(tags = tag)) # hack to make filterConceptList work

    page = ListPage(allConcepts, offset)
    page.data = makeConceptListTranslations(page.data, request.user, False)

    allTags = Tag.objects.all()
    return render_response(request, 'tag_detail.html', {'tag': tag, 'list': page, 'allTags':allTags})


def tagLookup(request):
    query = request.REQUEST['q']
    query= query.lstrip('-')
    minLen = 2

    results = searchExistingTags(query, minLen)
    tooShort = len(query) < minLen
    return render_response(request, 'tag_lookup.html', {'tags':results, 'tooShort':tooShort})

def tagFilterAdd(request):
    tagIDstr = request.REQUEST["tagFilter_id"]
    tagName = request.REQUEST["tagFilter_name"]

    if len(tagIDstr) > 0:
        tag = get_object_or_404(Tag, pk = tagIDstr)
        isReverse = False
    else :
        isReverse =tagName.startswith('-')
        tagName = tagName.lstrip('-')
        tag = getOrNoneTag(tagName)


    if (tag != None):
        f = TagFilter(request)
        if isReverse :
            f.addFilterEx(tag)
        else :
            f.addFilterInc(tag)
        f.save(request)
    else :
        request.session["message"] = "Nav taga: " + tagName

    return redirect_to_next(request)

def tagFilterRemove(request, tag_id):
    tag = get_object_or_404(Tag, pk=tag_id)
    f = TagFilter(request)
    f.removeFilter(tag)
    f.save(request)
    return redirect_to_next(request)

